/*
** Author :         Aico Peter van Vuuren
**                  Tinbergen Insitute Amsterdam
**                  Free University, Department of Economics, Amsterdam
** Last Revised :
** Purpose :        Computes distributions and densities of wage offer
**                  and earnings distribution as described in Van Vuuren,
**                  Van den Berg and Ridder.
*/


Model::Model(const lambda, const delta, const mu1, const mu2, const sigma,
		const pie, const wmin, const b, const m)
{
	Lambda = lambda;
	Delta = delta;
	Kappa = Lambda ./ Delta;
	Mu1 = mu1;
	Mu2 = mu2;
	Sigma = sigma;
	Pie = pie;
	Wmin = wmin;
	B = b;
	M = m;
}

Model::Dfie(const w)
{
    decl i, result, factor, po, pb, cn, aresult, aabserr;
	
	cn = rows(vec(w));	
	result = zeros(cn,1);
	
	for(i=0; i<cn; i++)
	{
        W = w[i];
	   	if (w <= max(Wmin|B[0]))
		{
            result[i] = 0;
        }
		else
		{
            po = Ponder(w[i]);
            pb = Pboven(w[i]);
            QAG(Dpsi, po, pb, 1, &aresult, &aabserr);
			result[i] = aresult;
        }
    }
    return result;
}

Model::Dgie(const w)
{
    decl i, result, factor, po, pb, cn, aresult, aabserr;
	
	cn = rows(vec(w));
	result = zeros(cn,1);
	
	for(i=0; i<cn; i++)
	{
        W = w[i];
      	if (w[i] <= max(Wmin|B[0]))
		{
            result[i] = 0;
        }
		else
		{
            po = Ponder(w[i]);
            pb = Pboven(w[i]);
            QAG(Dpsig, po, pb, -1, &aresult, &aabserr);
			result[i] = aresult;
        }
    }
    return result;
}

Model::Fie(const w)
{
    decl i, result, factor, po, pb, cn, aresult, aabserr;
	
	cn = rows(vec(w));
	result = zeros(cn,1);
	
	for(i=0; i<cn; i++)
	{
        W = w[i];
        if ((w[i] .<= Wmin) .|| (w[i] .<= B[0]))
		{
            result[i] = 0;
        }
		else
		{
            po = Ponder(w[i]);
            pb = Pboven(w[i]);
            QAG(Psi, po, pb, 1, &aresult, &aabserr);
			result[i] = aresult + Cumj(po);
        }
    }
    return result;
}

Model::Gie(const w)
{
    decl i, result, factor, po, pb, cn, aresult, aabserr;
	
	cn = rows(vec(w));
	result = zeros(cn,1);
	
	for(i=0; i<cn; i++)
	{
        W = w[i];
        if ((w[i] .<= Wmin) .|| (w[i] .<= B[0]))
		{
            result[i] = 0;
        }
		else
		{
            po = Ponder(w[i]);
            pb = Pboven(w[i]);
            QAG(Psig, po, pb, 1, &aresult, &aabserr);
			result[i] = aresult + Cumj(po);
        }
    }
    return result;
}
					   
Model::Denf(const w, const p)
{
    decl result, i,j;

	decl cm = rows(p);
	decl cn = columns(p);
	
	if ((cm .== 0) .&& (cn .== 0))
	{
		return Denf1(w,p);
	}
	
    result = zeros(cm, cn);

	for(i=0;i<cm;i++)
	{
        for(j=0;j<cn;j++)
		{
            result[i][j] = Denf1(w,p[i][j]);
        }
    }
    return result;
}
					   
Model::Deng(const w, const p)
{
    decl result, i,j;

	decl cm = rows(p);
	decl cn = columns(p);
	
	if ((cm .== 0) .&& (cn .== 0))
	{
		return Deng1(w,p);
	}
	
    result = zeros(cm, cn);

	for(i=0;i<cm;i++)
	{
        for(j=0;j<cn;j++)
		{
            result[i][j] = Deng1(w,p[i][j]);
        }
    }
    return result;
}

Model::Cumf(const w, const p)
{
    decl result, i,j;

	decl cm = rows(p);
	decl cn = columns(p);
	
	if ((cm .== 0) .&& (cn .== 0))
	{
		return Cumf1(w,p);
	}
	
    result = zeros(cm, cn);

	for(i=0;i<cm;i++)
	{
        for(j=0;j<cn;j++)
		{
            result[i][j] = Cumf1(w,p[i][j]);
        }
    }
    return result;
}

Model::Cumg(const w, const p)
{
    decl result, i,j;

	decl cm = rows(p);
	decl cn = columns(p);
	
	if ((cm .== 0) .&& (cn .== 0))
	{
		return Cumg1(w,p);
	}
	
    result = zeros(cm, cn);

	for(i=0;i<cm;i++)
	{
        for(j=0;j<cn;j++)
		{
            result[i][j] = Cumg1(w,p[i][j]);
        }
    }
    return result;
}

Model::Cumf1(const w, const p)
{
    decl c;
	
    if (B[2] < Wmin)
        return Cumf13(w,p,Wmin);
	
    if ((p > (B[2] - M[0] .* B[0]) ./ (1-M[0]))
        .&& (p > (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1])))
		return Cumf13(w,p,B[2]);
		
	if (Wmin > B[1])
	{
    	if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))
        	return Cumf13(w,p,B[2]);
			
       	return Cumf12(w,p,Wmin);
	}
 
	if ((p > ((M[0] + M[1]) .* B[1] - M[0] .* B[0])./(M[1])) .&&
            			(p < (B[2] - (M[0]+M[1]).*B[1] )./(1-M[0]-M[1])))
    	return Cumf12(w,p,B[1]);
				
	if(Wmin > B[0])
	{
        if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))		
        	return Cumf13(w,p,B[2]);
		if (p > ((M[0] + M[1]) .* B[1] - M[0] .* Wmin)./(M[1]))
        	return Cumf12(w,p,B[1]);
						
		return Cumf11(w,p,Wmin);
	}
    return Cumf11(w,p,B[0]);
}

Model::Cumg1(const w, const p)
{
    decl c;
	
    if (B[2] < Wmin)
        return Cumg13(w,p,Wmin);
	
    if ((p > (B[2] - M[0] .* B[0]) ./ (1-M[0]))
        .&& (p > (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1])))
		return Cumg13(w,p,B[2]);
		
	if (Wmin > B[1])
	{
    	if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))
        	return Cumg13(w,p,B[2]);
			
       	return Cumg12(w,p,Wmin);
	}
 
	if ((p > ((M[0] + M[1]) .* B[1] - M[0] .* B[0])./(M[1])) .&&
            			(p < (B[2] - (M[0]+M[1]).*B[1] )./(1-M[0]-M[1])))
    	return Cumg12(w,p,B[1]);
				
	if(Wmin > B[0])
	{
        if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))		
        	return Cumg13(w,p,B[2]);
		if (p > ((M[0] + M[1]) .* B[1] - M[0] .* Wmin)./(M[1]))
        	return Cumg12(w,p,B[1]);
						
		return Cumg11(w,p,Wmin);
	}
    return Cumg11(w,p,B[0]);
}

Model::Deng1(const w, const p)
{
    decl c;
    if (B[2] < Wmin)
        return Cumg13(w,p,Wmin);
	
    if ((p > (B[2] - M[0] .* B[0]) ./ (1-M[0]))
        .&& (p > (B[2] - (M[0]+M[1]).*B[1])
		./ (1-M[0]-M[1])))
            return Deng13(w,p,B[2]);
			
    // else
	if (Wmin > B[1])
	{
            if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))
                return Deng13(w,p,B[2]);
            else
                return Deng12(w,p,Wmin);
	}
	// else
	
    if ((p > ((M[0] + M[1]) .* B[1] - M[0] .* B[0])./(M[1])) .&&
            (p < (B[2] - (M[0]+M[1]).*B[1] )./(1-M[0]-M[1])))
                 return Deng12(w,p,B[1]);
	// else
	
    if (Wmin > B[0])
	{
            if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))
                return Deng13(w,p,B[2]);
			// else
			if (p > ((M[0] + M[1]) .* B[1] - M[0] .* Wmin)./
							(M[1]))
                return Deng12(w,p,B[1]);
            // else
            return Deng11(w,p,Wmin);
    }
	
    return Deng11(w,p,B[0]);
}					  
   
Model::Cumf11(const w, const p, const wmin)
{
    if (p < (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1]))
        return Cumf11a(w,p,wmin);
		
    return Cumf11b(w,p,wmin);
}
	 
Model::Deng11(const w,const p,const wmin)
{
    if (p < (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1]))
        return Deng11a(w,p,wmin);
    else
        return Deng11b(w,p,wmin);
}
		
Model::Cumg11(const w,const p,const wmin)
{
    if (p < (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1]))
	{
        return Cumg11a(w,p,wmin);
	}
    else
	{
        return Cumg11b(w,p,wmin);
	}
}

Model::Cumf11a(const w, const p, const wmin)
{
    decl c, c1, c2;
    c1 = M[0] ./ ((M[0] + M[1]) .* (1+Kappa)^2);
    c2 = M[0] ./ ((1+Kappa)^2);
    c = (1+Kappa)./Kappa;

    if (p < (B[1] - c1 .* wmin)./ (1-c1))
	{
        if (w < wmin)
			return 0;
		if (w < p - (1 ./ ((1+Kappa)^2)).*(p-wmin))
           	return c .* (1-sqrt((p-w)./((p-wmin))));
			
       	return 1;
	}
	
    if (p < (B[2] - c2.*wmin) ./ (1-c2))
	{
       	if (w < wmin)
	       	return 0;
		if (w < p - (p-B[1]).*((M[0]+M[1])./M[0]))	
            return c .* (1-sqrt((p-w)./((p-wmin))));		
		if (w < B[1])		
            return c .* (1-sqrt(((p-B[1])./(p-wmin))
										.*((M[0]+M[1])./M[0])));
		if (w < p - (p-wmin).*(M[0]./(M[0]+M[1]))
											./ ((1+Kappa)^2))
			return c .* (1-sqrt(((p-w)./(p-wmin)).*
											((M[0]+M[1])./M[0])));
						
        return 1;
	
	}
    	
	if (w < wmin)
    	return 0;
	if (w < p - (p-B[1]).*((M[0]+M[1])./M[0]))
       	return c .* (1-sqrt((p-w)./((p-wmin))));
	if (w < B[1])
		return c .* (1-sqrt(((p-B[1])./(p-wmin)).*((M[0]+M[1])
													./M[0])));    		
	if (w < p - (p-B[2])./ (M[0]+M[1]))
        return (c .* (1-sqrt(((p-w)./(p-wmin)).*((M[0]+M[1])
													./M[0]))));
	if (w < B[2])
		return c .* (1-sqrt((p-B[2])./((p-wmin).*M[0])));
	if (w < p - (((p-wmin).*M[0])./((1+Kappa)^2)))
    	return c .* (1-sqrt((p-w)./((p-wmin).*M[0])));
   
	return 1;
}
							 
Model::Cumg11a(const w,const p,const wmin)
{
    decl d, d1, d2, c1, c2;
    d1 = M[0] ./ ((M[1] + M[1]) .* (1+Kappa)^2);
    d2 = M[0] ./ ((1+Kappa)^2);

    if (p < (B[1] - d1 .* wmin)./ (1-d1))
	{
    	d = 1 ./Kappa;
        if (w < wmin)
            return 0;
		if (w < p - (1 ./((1+Kappa)^2)).*(p-wmin))
			return d .* (sqrt((p-wmin)./((p-w)))-1);
			
        return 1;
	}
    
	if (p < (B[2] - d2.*wmin) ./ (1-d2))
	{
      	c1 = (1 ./ (1+Kappa)) .* (M[0] + M[1] .* sqrt(((p-B[0])./(p-B[1]))
            	.*((M[0])./(M[0]+M[1]))));
       	d = 1 ./ (M[0]+M[1]-c1);
			
		if (w < wmin)
        	return 0;
		if (w < p - (p-B[1]).*((M[0]+M[1])./M[0]))
   			return ((M[0] .* d)./(1+Kappa)) .* (sqrt((p-wmin)
										./(p-w))-1);
		if (w < B[1])
			return d .* (c1 .* (sqrt(((p-wmin)./(p-B[1])).*((M[0])./(M[0]
				+M[1])))-1) - (M[1]./(1+Kappa)).*((sqrt(((p-wmin)./(p-
				B[1])).*((M[0])./(M[0]+M[1])))-1)./(sqrt(((p-B[1])
				./(p-wmin)).*((M[0]+M[1])./(M[0]))))));
		if (w < p - (p-wmin).*(M[0]./(M[0]+M[1])) ./
									((1+Kappa)^2))
			return d .* (c1 .* (sqrt(((p-wmin)./(p-w))
                .*((M[0])./(M[0]+M[1])))-1) - (M[1]./(1+Kappa))
                .*((sqrt(((p-wmin)./(p-B[1])).*((M[0])./(M[0]+M[1])))
              	-1)./(sqrt(((p-w)./(p-wmin)).*((M[0]+M[1])./(M[0]))))));
						
    	return 1;
	}

  
    c1 = (1 ./(1+Kappa)) .* (M[0] + M[1] .* sqrt(((p-B[0])
			./(p-B[1])) .*((M[0])./(M[0]+M[1]))));
   	c2 = (1 ./(1+Kappa)) .* (M[0] + M[1].*sqrt(((p-B[0])./(p-B[1]))
            	.*(M[0]./(M[0]+M[1])))
                + (1-M[0]-M[1]) .* sqrt(((p-B[0]).*M[0])./(p-B[2])));
   	d = M[1] ./ ((1+Kappa).*(1-c2));
   	if (w < wmin)
    	return 0;	
	if (w < p - (p-B[1]).*(M[0]./(M[0]+M[1])))
   		return M[0].* d.*(sqrt((p-B[0])./(p-w))-1);
	if (w < B[1])			
      	return (1 ./(1-c2)) .* (-c1 + ((M[0]+M[1])./(1+Kappa)).*sqrt(((p-
				wmin)./(p-B[1])).*(M[0]./(M[0]+M[1]))));
	if (w < p - (p-B[2])./ (M[0]+M[1]))
		return (1 ./ (1-c2)) .* (-c1 + ((M[0]+M[1])./(1+Kappa))
                    .*sqrt(((p-wmin)./(p-w)).*(M[0]./(M[0]+M[1]))));
	if (w < B[2])
		return (1 ./(1-c2)) .* (-c2 + (1 ./(1+Kappa)) .* sqrt(((p-wmin)
									.*M[0]) ./(p-B[2])));					
	if (w < p - (((p-wmin).*M[0])./((1+Kappa)^2)))
        return (1 ./(1-c2)) .* (-c2 + (1 ./(1+Kappa)) .*
								sqrt(((p-wmin).*M[0]) ./(p-w)));
								
	return 1;
}
        
			
Model::Deng11a(const w,const p,const wmin)
{
    decl d, d1, d2, c1, c2;
    d1 = M[0] ./ ((M[0] + M[1]) .* (1+Kappa)^2);
    d2 = M[0] ./ ((1+Kappa)^2);

    if (p < (B[1] - d1 .* wmin)./ (1-d1))
	{
        d = 1 ./(2 .*Kappa);
        if (w < wmin)
            return 0;
        if (w <= p - (1 ./((1+Kappa)^2)).*(p-wmin))
            return d .* ((sqrt(p-wmin))./((p-w)^(3/2)));
			
		return 0;
	}
	// else
    if (p < (B[2] - d2.*wmin) ./ (1-d2))
	{
        c1 = (1 ./(1+Kappa)) .* (M[0] + M[1] .* sqrt(((p-B[0])./(p-B[1]))
            .*((M[0])./(M[0]+M[1]))));
        d = 1 ./ (2 .*(M[0]+M[1]-c1));
        if (w < wmin)
            return 0;
        if (w < p - (p-B[1]).*((M[0]+M[1])./M[0]))
            return ((M[0])./(1+Kappa)) .* d .* ((sqrt(p-wmin))./((p-w)^(3/2)));
        if (w < B[1])
            return 0;
        if (w <= p - (p-wmin).*(M[0]./(M[0]+M[1])) ./ ((1+Kappa)^2))
            return d .* ((sqrt((p-wmin).*((M[0])./(M[0]+M[1]))))
                ./((p-w)^(3/2))).* (c1 - (M[1]./(1+Kappa)).*
                (sqrt(((p-wmin)./(p-B[1])).*((M[0])./(M[0]+M[1])))-1));
		return 0;
	}
    // else
    c1 = (1 ./(1+Kappa)) .* (M[0] + M[1] .* sqrt(((p-B[0])./(p-B[1]))
            .*((M[0])./(M[0]+M[1]))));
    c2 = (1 ./(1+Kappa)) .* (M[0] + M[1].*sqrt(((p-B[0])./(p-B[1]))
            .*(M[0]./(M[0]+M[1])))
                + (1-M[0]-M[1]) .* sqrt(((p-B[0]).*M[0])./(p-B[2])));
    d = M[0] ./ (2 .*(1+Kappa).*(1-c2));
    if (w < wmin)
    	return 0;
    if (w <= p - (p-B[1]).*((M[0]+M[1])./M[0]))
            return M[0].* d.*((sqrt(p-wmin))./((p-w)^(3/2)));
    if (w < B[1])
            return 0;
    if (w <= p - (p-B[2])./ (M[0]+M[1]))
            return (1 ./(2 .*(1-c2))) .* ((sqrt((p-wmin).*((M[0])./
				(M[0]+M[1])))) ./((p-w)^(3/2))).*
				(c1 - (M[1]./(1+Kappa)).*
                (sqrt(((p-wmin)./(p-B[1])).*((M[0])./(M[0]+M[1])))-1));
	if (w < B[2])
            return 0;
    if (w <= p - (((p-wmin).*M[0])./((1+Kappa)^2)))
            return (1 ./(2 .*(1-c2))) .* ((sqrt((p-B[0]).*M[0]))./((p-w)^(3/2)))
                .* (c2 - (M[1].*(sqrt(((p-B[0])./(p-B[1])).*
                ((M[0])./(M[0]+M[1])))-1) + (1-M[0]-M[1])
                .*(sqrt(((p-B[0]).*M[0])./(p-B[2]))-1))./(1+Kappa));
				
    return 0;
}


Model::Cumf11b(const w,const p,const wmin)
{
    decl c, c1;
    c1 = (1 - (1 ./((1+Kappa)^2)));
    c = (1+Kappa)./Kappa;
	
    if (p > (B[2] - wmin ./ ((1+Kappa)^2)) ./ c1)
	{
        if (w < wmin)
            return 0;
		if (w < p - (p-B[2]) ./ M[0])
            return c .* (1-sqrt((p-w)./((p-wmin))));
		if (w < B[2])
       		return c .* (1-sqrt((p-B[2])./((p-wmin).*M[0])));
		if (w < p - ((p-wmin)./((1+Kappa)^2)))
   			return c .* (1-sqrt((p-w)./((p-wmin).*M[0])));
			
		return 1;
	}
  
    if (w < wmin)
        return 0;
	if (w < p - (((p-wmin))./((1+Kappa)^2)))
       	return c .* (1-sqrt((p-w)./((p-wmin))));
			
	return 1;
}

Model::Cumg11b(const w, const p, const wmin)
{
    decl d, d1, c4;
	
    d1 = (1 - (1 ./((1+Kappa)^2)));
    d = (1+Kappa)./Kappa;
    if (p > (B[2] - wmin ./ ((1+Kappa)^2)) ./ d1)
	{
        c4 = (1 ./(1+Kappa)) .* (M[0] + (1-M[0]).*sqrt(((p-B[0]).*M[0])
        		./(p-B[2])));
        d = (M[1])./((1+Kappa).*(1-c4));
		
        if (w < wmin)
            return 0;
		if (w < p - (p-B[2]) ./ M[0])
            return d .* (sqrt((p-wmin)./(p-w))-1);	
		if (w < B[2])
       		return (1 ./(1-c4)).* (c4.*(sqrt(((p-B[0]).*M[0])
						./(p-B[2]))-1) - ((1-M[1])./(1+Kappa))
            			.*((sqrt(((p-B[0]).*M[0])./(p-B[2]))-1)
            			./(sqrt((p-B[2])./((p-B[0])).*M[0]))));
		if (w < p - ((p-wmin)./((1+Kappa)^2)))
   			return (1 ./(1-c4)).* (c4.*(sqrt(((p-B[0]).*M[0])
						./(p-w))-1) - ((1-M[0])./(1+Kappa))
            			.*((sqrt(((p-B[0]).*M[0])./(p-B[2]))-1)
            			./(sqrt((p-w)./((p-B[0])).*M[0]))));
				
		return 1;
	}
   
	
    d = 1 ./ Kappa;
    if (w < wmin)
	    return 0;
	if (w < p - (((p-wmin))./((1+Kappa)^2)))
        return d .* (sqrt((p-wmin)./(p-w)) -1);

	return 1;
}

Model::Deng11b(const w,const p,const wmin)
{
    decl d, d1, c4;
    d1 = (1 - (1 ./((1+Kappa)^2)));
    d = (1+Kappa)./Kappa;
	
    if (p > (B[2] - wmin ./ ((1+Kappa)^2)) ./ d1)
	{
        c4 = (1 ./(1+Kappa)) .* (M[0] + (1-M[0]).*sqrt(((p-B[0]).*M[0])
        					./(p-B[2])));
        d = (M[1])./(2 .*(1+Kappa).*(1-c4));
        if (w < wmin)
            return 0;
        if (w <= p - (p-B[2]) ./ M[0])
            return d .* ((sqrt(p-B[0]))./((p-w)^(3/2)));
        if (w < B[2])
            return 0;
        if (w <= p - ((p-wmin)./((1+Kappa)^2)))
            return (1 ./(2 .*(1-c4))) .* ((sqrt((p-B[0])).*M[0])
				./((p-w)^(3/2))).*(c4 - ((1-M[0])./(1+Kappa))
	                .*(sqrt(((p-B[0]).*M[0])./(p-B[2]))-1));
        return 0;
	}
    
    d = 1 ./(2 .*Kappa);
    if (w < wmin)
    	return 0;
    if (w <= p - (((p-wmin))./((1+Kappa)^2)))
        return d .* ((sqrt(p-B[0]))./(p-w)^(3/2));

	return 0;
}

Model::Cumf12(const w,const p,const wmin)
{
    decl c;
    c = (1+Kappa)./Kappa;

    if (p > (B[2] - ((M[0]+M[1]).*wmin)./((1+Kappa)^2))
                ./(1 - (M[0]+M[1])./((1+Kappa)^2)))
	{
        if (w < wmin)
            return 0;
		if (w < p - (p-B[2])./(M[0]+M[1]))
            return c .* (1-sqrt((p-w)./(p-wmin)));
		if (w < B[2])
            return c .* (1-sqrt((p-B[2])./((p-wmin).*(M[0]+M[1]))));
		if (w < p - (1 ./ ((1+Kappa)^2)) .* (p-wmin) .* (M[0]+M[1]))
            return c .* (1-sqrt((p-w)./((p-wmin).*(M[0]+M[1]))));
			
       	return 1;

	}
   
    if (w < wmin)
    	return 0;
	if (w < p - (1 ./ ((1+Kappa)^2)) .* (p-wmin))
       	return c.* (1-sqrt((p-w)./(p-wmin)));
     
	return 1;
}


Model::Cumg12(const w,const p, const wmin)
{
    decl d, c0;

    if (p > (B[2] - ((M[0]+M[1]).*wmin)./((1+Kappa)^2))
                ./(1 - (M[0]+M[1])./((1+Kappa)^2)))
	{
        c0 =(1 ./(1+Kappa)).*(M[0]+M[1]+(1-M[0]-M[1]).*sqrt(((p-wmin)
            .*(M[0]+M[1]))./(p-B[2])));
        d = (M[0]+M[1])./((1+Kappa).*(1-c0));
        if (w < wmin)
            return 0;			
		if (w < p - (p-B[2])./(M[0]+M[1]))
			return d .* (sqrt((p-wmin)./(p-w))-1);
       	if (w < B[2])
            return (1 ./(1-c0)).*(c0.*(sqrt(((p-wmin).*(M[0]+M[1]))./(p-B[2]))
            			-1) - ((1-M[0]-M[1])./(1+Kappa)).*((sqrt(((p-wmin).*(M[0]+M[1]))
            			./(p-B[2]))-1)./(sqrt((p-B[2])./((p-wmin).*(M[0]+M[1]))))));
		if (w < p - (1 ./((1+Kappa)^2)) .* (p-wmin) .* (M[0]+M[1]))
            return (1 ./(1-c0)).*(c0.*(sqrt(((p-wmin).*(M[1]+M[0]))./(p-w))
         				-1) - ((1-M[0]-M[1])./(1+Kappa)).*((sqrt(((p-wmin).*(M[0]+M[1]))
           				./(p-B[2]))-1)./(sqrt((p-w)./((p-wmin).*(M[0]+M[1]))))));
					
		return 1;
	}
    
	d = 1 ./Kappa;
    if (w < wmin)
		return 0;
      
	if (w < p - (1 ./((1+Kappa)^2)) .* (p-wmin))
         return d.* (sqrt((p-wmin)./(p-w))-1);

	return 1;
}

Model::Deng12(const w,const p,const wmin)
{
    decl d, c0;

    if (p > (B[2] - ((M[0]+M[1]).*wmin)./((1+Kappa)^2))
                ./(1 - (M[0]+M[1])./((1+Kappa)^2)))
	{
        c0 =(1 ./(1+Kappa)).*(M[0]+M[1]+(1-M[0]-M[1]).*sqrt(((p-wmin)
            .*(M[0]+M[1]))./(p-B[2])));
        d = (M[0]+M[1])./(2 .*(1+Kappa).*(1-c0));
        if (w < wmin)
            return 0;
        if (w <= p - (p-B[2])./(M[0]+M[1]))
			return d .* ((sqrt(p-wmin))./((p-w)^(3/2)));
		if (w < B[2])
            return 0;
        if (w <= p - (1 ./((1+Kappa)^2)) .* (p-wmin) .* (M[0]+M[1]))
            return (1 ./(2 .*(1-c0))) .* ((sqrt((p-wmin).*(M[0]+M[1])))./((p-w)^(3/2)))
            .* (c0 - ((1-M[0]-M[1])./(1+Kappa)).*(sqrt(((p-wmin).*
				(M[0]+M[1]))./(p-B[2]))-1));
        
        return 0;
	}
   
    d = 1 ./(2 .*Kappa);
    if (w < wmin)
        return 0;
    if (w <= p - (1 ./((1+Kappa)^2)) .* (p-wmin))
		return d.* ((sqrt(p-wmin))./((p-w)^(3/2)));
		
	return 0;
}

Model::Cumf13(const w, const p, const wmin)
{
    decl c;
    c = (1+Kappa)./Kappa;
    if (w < wmin)
	{
        return 0;
    }
	else
	{
		if (w < p - (1 ./((1+Kappa)^2)) .* (p-wmin))
		{
        	return c.*(1-sqrt((p-w)./(p-wmin)));
		}
	}
	return 1;
}


Model::Cumg13(const w,const p,const wmin)
{
    decl c;
    c = 1 ./Kappa;
    if (w < wmin)
        return 0;

	// else
	if (w < p - (1 ./((1+Kappa)^2)) .* (p-wmin))
        return c.*(sqrt((p-wmin)./(p-w))-1);
    	
	// else
    return 1;
}


Model::Deng13(const w, const p, const wmin)
{
    decl c;
    c = 1 ./(2 .*Kappa);
	
    if (w < wmin)
        return 0;
    if (w <= p - (1 ./((1+Kappa)^2)) .* (p-wmin))
        return c.*((sqrt(p-wmin))./((p-w)^(3/2)));
		
	return 0;
}		
		
Model::Denf1(const w, const p){
    decl c;
    if (B[2] < Wmin)
        return Denf13(w,p,Wmin);

	
    if ((p > (B[2] - M[0] .* B[0]) ./ (1-M[0]))
        .&& (p > (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1])))
            return Denf13(w,p,B[2]);
			
	if (Wmin > B[1])
	{
    	if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))
			return Denf13(w,p,B[2]);

		return Denf12(w,p,Wmin);
    }      
    
	if ((p > ((M[0] + M[1]) .* B[1] - M[0] .* B[0])
					./(M[1])) .&&
            	(p < (B[2] - (M[0]+M[1]).*B[1] )./(1-M[0]-M[1])))
    	        	return Denf12(w,p,B[1]);
			
	if (Wmin > B[0])
	{
    	if (p > (B[2] - (M[0]+M[1]).*Wmin) ./ (1-M[0]-M[1]))
			return Denf13(w,p,B[2]);
		if (p > ((M[0] + M[1]) .* B[1] - M[0] .* Wmin)
							./(M[1]))
         	return Denf12(w,p,B[1]);
        
        return Denf11(w,p,Wmin);
	}
    return Denf11(w,p,B[0]);
}

Model::Denf11(const w, const p, const wmin)
{
    if (p < (B[2] - (M[0]+M[1]).*B[1]) ./ (1-M[0]-M[1]))
        return Denf11a(w,p,wmin);

    return Denf11b(w,p,wmin);
}

Model::Denf11a(const w, const p, const wmin)
{
    decl c, c1, c2;
	
    c1 = M[0] ./ ((M[0] + M[1]) .* (1+Kappa)^2);
    c2 = M[0] ./ ((1+Kappa)^2);
    c = (1+Kappa)./Kappa;

    if (p < (B[2] - c1 .* wmin)./ (1-c1))
	{
        if (w < wmin)
			return 0;
		if (w <= p - (1 ./((1+Kappa)^2)).*(p-wmin))
           	return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));
			
       	return 0;
	}
    
	if (p < (B[2] - c2.*wmin) ./ (1-c2))
	{
		if (w < wmin)
           	return 0;
		if (w <= p - (p-B[1]).*((M[0]+M[1])./M[0]))
            return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));			
		if (w < B[1])
			return 0;
		if (w <= p - (p-wmin).*(M[0]./(M[0]+M[1]))
											./ ((1+Kappa)^2))
			return 0.5 .* c ./ (sqrt((p-w).*(p-wmin).*(M[0]
											./(M[0]+M[1]))));
            return 0;
	}
    if (w < wmin)
    	return 0;
	if (w <= p - (p-B[1]).*((M[0]+M[1])./M[0]))
		return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));
	if (w < B[1])
         return 0;		
	if (w <= p - (p-B[2])./(M[0]+M[1]))
		return 0.5 .* c ./ (sqrt((p-w).*(p-wmin).*(M[0]
												./(M[0]+M[1]))));			
	if (w < B[2])
		return 0;						
	if (w <= p - (((p-wmin).*M[0])./((1+Kappa)^2)))							
		return 0.5 .* c ./ (sqrt((p-w).*(p-wmin) .* M[0]));

	return 0;
}

Model::Denf11b(const w,const p, const wmin)
{
    decl c, c1;
    c1 = (1 - (1 ./((1+Kappa)^2)));
    c = (1+Kappa)./Kappa;
    if (p > (B[2] - wmin ./ ((1+Kappa)^2)) ./ c1)
	{
        if (w < wmin)
            return 0;
        if (w <= p - (p-B[2]) ./ M[0])
            return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));
		if (w < B[2])
	        return 0;		
		if (w <= p - ((p-wmin)./((1+Kappa)^2)))
           	return 0.5 .* c ./ (sqrt((p-w).*(p-wmin) .* M[0]));

		 return 0;
	}
    if (w < wmin)
            return 0;
	if (w <= p - (((p-wmin))./((1+Kappa)^2)))
			return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));

	return 0;
}
		
Model::Denf12(const w, const p,const wmin)
{
    decl c;
    c = (1+Kappa)./Kappa;

    if (p > (B[2] - ((M[0]+M[1]).*wmin)./((1+Kappa)^2))
                ./(1 - (M[0]+M[1])./((1+Kappa)^2)))
	{
        if (w < wmin)
            return 0;
        if (w <= p - (p-B[2])./(M[0]+M[1]))
            return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));
		if (w < B[2])
            return 0;
		if (w <= p - (1 ./((1+Kappa)^2)) .* (p-wmin) .* (M[0]+M[1]))
			return 0.5 .* c ./ (sqrt((p-w).*(p-wmin) .* (M[0]+M[1])));

		return 0;
    }
	
    if (w < wmin)
    	return 0;
	if (w <= p - (1 ./((1+Kappa)^2)) .* (p-wmin))
       	return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));
		
	return 0;
}
  
Model::Denf13(const w, const p, const wmin)
{
    decl c;
    c = (1+Kappa)./Kappa;

    if (w < wmin)
        return 0;
	if (w <= p - (1 ./ ((1+Kappa)^2)) .* (p-wmin))
       	return 0.5 .* c ./ (sqrt((p-w).*(p-wmin)));
		
    return 0;	
}

Model::Pboven(const w)
{
    decl c, wmin,result;
    c = (1+Kappa)./Kappa;
    if (w < B[1])
	{
        wmin = min((B[0]~Wmin)');
        result = ((((M[0]+M[1]).* B[1] - M[0].*wmin)./M[1])|
                ((B[2] - M[0].*wmin)./(1-M[0])));
        return min(result);
	}
    else
	{
		if (w < B[2])
		{
        	wmin = min(B[0]|Wmin);
        	return (B[2]-(M[0]+M[1]).*wmin)./(1-M[0]-M[1]);
		}
    	else
		{
        	return 100000;
    	}
	}
}

Model::Cumj(const p)
{
    decl wmin;
    wmin = max((Wmin~B[0])');
    return (Cumj1(p)-Cumj1(wmin)) ./ (1-Cumj1(wmin));
}

Model::Cumj1(const p)
{
    return Pie.*Cumj2(p) + (1-Pie).*Cumj3(p);
}

Model::Cumj2(const p)
{
    return probn((log(p)-Mu1)./Sigma);
}

Model::Cumj3(const p)
{
    return probn((log(p)-Mu2)./Sigma);
}

	 
Model::Psi(const p)
{
    return Cumf(W,p) .* Jie(p);
}	   

Model::Jie1(const p)
{
    return Pie .* Jie2(p) + (1-Pie).*Jie3(p);
}

Model::Jie2(const p)
{
    return (densn((log(p)-Mu1)./Sigma) .* (1 ./(Sigma.*p)));
}

Model::Jie3(const p)
{
    return (densn((log(p)-Mu2)./Sigma) .* (1 ./(Sigma.*p)));
}

Model::Jie(const p)
{
    decl wmin;
    wmin = max((Wmin~B[0])');
    return Jie1(p) ./ (1-Cumj1(wmin));
}

Model::Dpsi(const p)
{
    return  Denf(W,p) .* Jie(p);
}

Model::Dponder(const w)
{
    return ones(rows(w),1);
}

Model::Psiu(const p)
{
    return Beru(p) .* Jie1(p);
}
	  
Model::Puu()
{
    return Bers() + Berf();
}		
			/*
Model::Puu1()
{
    return Puu() - U;
}		  

Model::Berkappa(const k)
{
	decl avScore;
	decl result = k;
    while (fabs(Puu1(k)) > 0.0001)
	{
		Num1Derivative(Puu1, result, avScore);
        result = result - Puu1(result) ./ avScore;
	}
    return result;
}		  */

Model::Psifie(const p, const t)
{
    decl result, i;
    result = 0;

	for(i=0;i<rows(B)-1;i++)
	{
        result = result + (Cumf(B[i],p) .< 1) .*
                 (M[i] .* exp(-Lambda.*t.*(1-Cumf(B[i],p))));
    }
    result = result + (Cumf(B[2],p) .< 1) .*
                 ((1-M[0]-M[1]) .* exp(-Lambda.*t.*(1-Cumf(B[2],p))));
    return Jie1(p).*result;
}

Model::Psifieb(const p, const t)
{
    decl result;
    result = result + (Cumf(B[2],p) .< 1) .*
                 (exp(-Lambda.*t.*(1-Cumf(B[2],p))));
    return Jie1(p).*result;
}

Model::Beru(const p)
{
    decl result, i;
    
    result = 0;
	for(i=0;i<rows(B);i++)
	{
        result = result + M[i] ./ (1+Kappa.*(1-Cumf(B[i],p)));
	}
    result = result + (1-M[0]-M[1]) ./ (1+Kappa.*(1-Cumf(B[2],p)));
    return result;
}

Model::Psig(const p)
{
    return Cumg(W,p) .* Jie(p);
}
		   
Model::Dpsig(const p)
{
    return Deng(W,p) .* Jie(p);
}

Model::Bers()
{
    decl result;
    if (Wmin > B[2])
	{
        return Cumj1(Ponder(Wmin));
	}
	
    if (Wmin > B[1])
	{
        return ((M[0] + M[1]) .* Cumj1(Ponder(Wmin)) + M[2] .*
            Cumj1(Ponder(B[2])));
	}
    
	if (Wmin > B[0])
	{
        return (M[0] .* Cumj1(Ponder(Wmin))+ M[1] .* Cumj1(Ponder(B[1])) +
                M[2] .* Cumj(Ponder(B[2])));
	}
	
    return M[0] .* Cumj1(Ponder(B[0])) + M[1] .*
                Cumj1(Ponder(B[1])) + M[2] .* Cumj1(Ponder(B[2]));
}

Model::Beralpha()
{
    return Bers() ./ Puu();
}

Model::Berf()
{
    decl result, po1, po2, po3, pb1, pb2, pb3, hulp1, hulp2, hulp3, i,
            po, pb, aresult, aabserr;

    if (Wmin < B[1])
	{
        po1 = Ponder(B[0]);
        po2 = Ponder(B[1]);
        po3 = Ponder(B[2]);
        pb1 = Pboven(B[0]);
        pb2 = Pboven(B[1]);
        pb3 = Pboven(B[2]);
        hulp1 = po1+0.5.*(pb1-po1);
        hulp2 = po2+0.5.*(pb2-po2);
        hulp3 = po3+0.5.*(pb3-po3);

        result = (M[0]./(1+Kappa)).*(1-Cumj1(Pboven(B[0])));
        result = result + (M[1]./(1+Kappa)).*(1-Cumj1(Pboven(B[1])));
        result = result + (M[2]./(1+Kappa)).*(1-Cumj1(Pboven(B[2])));
		QAG(Psif1, po1, pb1, 1, &aresult, &aabserr);
        result = result + M[0].*(aresult);
		QAG(Psif2, po2, pb2, 1, &aresult, &aabserr);
        result = result + M[1].*(aresult);
		QAG(Psif3, po3, pb3, 1, &aresult, &aabserr);
        result = result + M[2].*(aresult);
        return result;
	}
	
    if (Wmin < B[1])
	{
        po1 = Ponder(Wmin);
        po2 = Ponder(B[1]);
        po3 = Ponder(B[2]);
        pb1 = Pboven(Wmin);
        pb2 = Pboven(B[1]);
        pb3 = Pboven(B[2]);
        hulp1 = po1+0.5.*(pb1-po1);
        hulp2 = po2+0.5.*(pb2-po2);
        hulp3 = po3+0.5.*(pb3-po3);

        result = (M[0]./(1+Kappa)).*(1-Cumj1(Pboven(Wmin)));
        result = result + (M[1]./(1+Kappa)).*(1-Cumj1(Pboven(B[1])));
        result = result + (M[2]./(1+Kappa)).*(1-Cumj1(Pboven(B[2])));
		QAG(Psif11, po1, pb1, 1, &aresult, &aabserr);
        result = result + M[0].*(aresult);
		QAG(Psif2, po2, pb2, 1, &aresult, &aabserr);
        result = result + M[1].*(aresult);
		QAG(Psif3, po3, pb3, 1, &aresult, &aabserr);
        result = result + M[2].*(aresult);
        return result;
	}
	
    if (Wmin < B[2])
	{
        po2 = Ponder(Wmin);
        po3 = Ponder(B[2]);
        pb2 = Pboven(Wmin);
        pb3 = Pboven(B[2]);
        hulp2 = po2+0.5.*(pb2-po2);
        hulp3 = po3+0.5.*(pb3-po3);

        result = ((M[0]+M[1])./(1+Kappa))
                           .*(1-Cumj1(Pboven(Wmin)));
        result = result + (M[2]./(1+Kappa))
                        .*(1-Cumj1(Pboven(B[2])));
		QAG(Psif11, po2, pb2, 1, &aresult, &aabserr);
        result = result + (M[0]+M[1]).*(aresult);
		QAG(Psif3, po3, pb3, 1, &aresult, &aabserr);
        result = result + M[2].*(aresult);	
        return result;
	}
    
    return ((1 ./(1+Kappa)).*(1-Cumj1(Ponder(Wmin))));
}
				
Model::Berpur()
{
    decl result;
	
    if (Wmin < B[0])
	{
        result = (M[0]./(1+Kappa)).*(1-Cumj1(Ponder(B[0])));
        result = result + (M[1]./(1+Kappa)).*(1-Cumj1(Ponder(B[1])));
        result = result + (M[2]./(1+Kappa)).*(1-Cumj1(Ponder(B[2])));
        return result;
	}
	
    if (Wmin < B[1])
	{
        result = (M[0]./(1+Kappa)).*(1-Cumj1(Ponder(Wmin)));
        result = result + (M[1]./(1+Kappa)).*(1-Cumj1(Ponder(B[1])));
        result = result + (M[2]./(1+Kappa)).*(1-Cumj1(Ponder(B[2])));
        return result;
	}
    if (Wmin < B[2])
	{
        result = ((M[0]+M[1])./(1+Kappa)).*(1-Cumj1(Ponder(Wmin)));
        result = result + (M[2]./(1+Kappa)).*(1-Cumj1(Ponder(B[2])));
        return result;
    }

    return (1 ./(1+Kappa)).*(1-Cumj1(Ponder(Wmin)));
}
								
Model::Psit1(const t)
{
    decl result, po1, po2, po3, pb1, pb2, pb3, hulp1, hulp2, hulp3,
			aresult, aabserr;

    if (Wmin < B[1])
	{
        po1 = Ponder(B[0]);
        po2 = Ponder(B[1]);
        po3 = Ponder(B[2]);
        pb1 = Pboven(B[0]);
        pb2 = Pboven(B[1]);
        pb3 = Pboven(B[2]);

        T = t;
		
        result = (M[0]./(1+Kappa)).* exp(-Lambda .* t)
                            .*(1-Cumj1(Pboven(B[0])));
        result = result + (M[1]./(1+Kappa)).*
                            exp(-Lambda .* t).*(1-Cumj1(Pboven(B[1])));
        result = result + (M[2]./(1+Kappa))
                        .* exp(-Lambda .* t).*(1-Cumj1(Pboven(B[2])));
		QAG(Psit1a, po1, pb1, 1, &aresult, &aabserr);
        result = result + M[0].*(aresult);
		QAG(Psit2a, po2, pb2, 1, &aresult, &aabserr);
        result = result + M[1].*(aresult);
		QAG(Psit3a, po3, pb3, 1, &aresult, &aabserr);
        result = result + M[2].*(aresult);
		
        return result;
	}
	
    if (Wmin < B[1])
	{
        po1 = Ponder(Wmin);
        po2 = Ponder(B[1]);
        po3 = Ponder(B[2]);
        pb1 = Pboven(Wmin);
        pb2 = Pboven(B[1]);
        pb3 = Pboven(B[2]);

        T = t;
        result = (M[0]./(1+Kappa)).* exp(-Lambda .* t)
                            .*(1-Cumj1(Pboven(Wmin)));
        result = result + (M[1]./(1+Kappa)).*
                            exp(-Lambda .* t).*(1-Cumj1(Pboven(B[1])));
        result = result + (M[2]./(1+Kappa))
                        .* exp(-Lambda .* t).*(1-Cumj1(Pboven(B[2])));
		QAG(Psit11a, po1, pb1, 1, &aresult, &aabserr);
        result = result + M[0].*(aresult);
		QAG(Psit2a, po2, pb2, 1, &aresult, &aabserr);
        result = result + M[1].*(aresult);
		QAG(Psit3a, po3, pb3, 1, &aresult, &aabserr);
        result = result + M[2].*(aresult);
        return result;
	}
	
    if (Wmin < B[2])
	{
        po2 = Ponder(Wmin);
        po3 = Ponder(B[2]);
        pb2 = Pboven(Wmin);
        pb3 = Pboven(B[2]);
        hulp2 = po2+0.5.*(pb2-po2);
        hulp3 = po3+0.5.*(pb3-po3);

        T = t;
        result = ((M[0]+M[1])./(1+Kappa)).*
                            exp(-Lambda .* t).*(1-Cumj1(Pboven(Wmin)));
        result = result + (M[2]./(1+Kappa))
                        .* exp(-Lambda .* t).*(1-Cumj1(Pboven(B[2])));
		QAG(Psit11a, po2, pb2, 1, &aresult, &aabserr);
        result = result + (M[0]+M[1]).*aresult;
		QAG(Psit3a, po3, pb3, 1, &aresult, &aabserr);
        result = result + M[2].*aresult;
        return result;
	}
	
    return ((1 ./(1+Kappa)) .* exp(-Lambda .* t).*(1-Cumj1(Ponder(Wmin))));
}

Model::Psit(const t)
{
	decl i, result;
	if (rows(t) .== 0)
		return Beralpha() + (1 ./ Puu()) .* Psit1(t);

	result = <>;
	
	for(i=0;i<rows(t);i++)
	{
    	result = result|(Beralpha() + (1 ./Puu()).* Psit1(t[i]));
	}
	return result;
}
  
Model::Psit1a(const p)
{
    return (Jie1(p) .* exp(-Lambda .* (1-Cumf(B[0],p)) .* T))
                ./ (1 + Kappa .* (1-Cumf(B[0], p)));
}

Model::Psit11a(const p)
{
    return (Jie1(p) .* exp(-Lambda .* (1-Cumf(Wmin,p)) .* T))
                ./ (1 + Kappa .* (1-Cumf(Wmin, p)));
}

Model::Psit2a(const p)
{
    return (Jie1(p) .* exp(-Lambda .* (1-Cumf(B[1],p)) .* T))
                ./ (1 + Kappa .* (1-Cumf(B[1], p)));
}

Model::Psit3a(const p)
{
    return (Jie1(p) .* exp(-Lambda .* (1-Cumf(B[2],p)) .* T))
                ./ (1 + Kappa .* (1-Cumf(B[2], p)));
}
	
Model::Psif1(const p)
{
    return Jie1(p)./ (1 + Kappa .* (1-Cumf(B[0], p)));
}

Model::Psif11(const p)
{
    return Jie1(p)./ (1 + Kappa .* (1-Cumf(Wmin, p)));
}

Model::Psif2(const p)
{
    return Jie1(p)./ (1 + Kappa .* (1-Cumf(B[1], p)));
}

Model::Psif3(const p)
{
    return Jie1(p)./ (1 + Kappa .* (1-Cumf(B[2], p)));
}

Model::Psif(const p)
{
    decl result, i;
    result = 0;
    for(i=0;i < rows(B)-1;i++)
	{
        result = result + M[i] .* (Cumf(B[i],p) .< 1) .*
            (1 ./(1+Kappa.*(1-Cumf(B[i],p))));
    }
    result = result + (1-M[1]-M[2]) .* (Cumf(B[3],p) .< 1) .*
            (1 ./(1+Kappa.*(1-Cumf(B[3],p))));
    return result.*Jie1(p);
}
				 
			 
Model::Ponder(const w)
{
	return w+1e-7;
}
			 

Model::Ponder1(const vP, const adFunc, const avScore, const mHess)
{
    adFunc[0] = Cumf(W,vP) - 1;
	return 1;
}

Model::GetLambda()
{
	return Lambda;
}

Model::GetDelta()
{
	return Delta;
}

Model::GetPie()
{
	return Pie;
}

Model::GetSigma()
{
	return Sigma;
}

Model::GetMu1()
{
	return Mu1;
}

Model::GetMu2()
{
	return Mu2;
}

Model::ModifyLambda(const lambda)
{
	Lambda = lambda;
	Kappa = Lambda ./ Delta;
}

Model::ModifyDelta(const delta)
{
	Delta = delta;
	Kappa = Lambda ./ Delta;
}

Model::ModifyPie(const pie)
{
	Pie = pie;
}

Model::ModifySigma(const sigma)
{
	Sigma = sigma;
}

Model::ModifyMu1(const mu1)
{
	Mu1 = mu1;
}

Model::ModifyMu2(const mu2)
{
	Mu2 = mu2;
}
			   